#!/bin/bash

OUTDIR=$1
SIZE=$2
N=$3
SCRIPT=$4

P=$((N/100))

mkdir $OUTDIR

./$SCRIPT.sh $N $SIZE > $OUTDIR/init.log 2>&1

echo timestamp run workers eic probability time > $OUTDIR/results.csv

# runs
for r in `seq 1 3`; do

	# parallel workers per gather
	for workers in 0 4; do

		# effective_io_concurrency
		for eic in 0 1 4 16 64 128; do

			# fraction of table to scan
			for p in 1 5 10 25 50 75; do

				if [ -c "stop" ]; then
					exit
				fi

				# clean page cache
				sudo ./drop-caches.sh

				m=$((N - P*p))
				s=$((RANDOM % m))
				e=$((s + P*p))

				echo "\timing on
set enable_seqscan = off;
set enable_indexscan = off;
set max_parallel_workers_per_gather = $workers;
set effective_io_concurrency = $eic;
SELECT * FROM (SELECT * FROM t WHERE a >= $s AND a < $e OFFSET 1000000000) foo;" > /tmp/query.sql

				echo "\timing on
set enable_seqscan = off;
set enable_indexscan = off;
set max_parallel_workers_per_gather = $workers;
set effective_io_concurrency = $eic;
EXPLAIN (SETTINGS) SELECT * FROM (SELECT * FROM t WHERE a >= $s AND a < $e OFFSET 1000000000) foo;" > /tmp/explain.sql

				ts=`date +%s`

				echo "========== $ts RUN $r EIC $eic WORKERS $workers ASYNC $async BUFFERS $buff SAMPLE $p ==========" >> $OUTDIR/explain.txt

				psql test < /tmp/explain.sql >> $OUTDIR/explain.txt 2>&1

				# run iostat in the background
				iostat -x -k 1 > iostat/$ts-run-$r-eic-$eic-workers-$workers-sample-$p.log 2>&1 &
				pid=$!

				psql test < /tmp/query.sql > /tmp/result.log 2>&1

				# kill the iostat running in background
				kill $pid > /dev/null 2>&1

				cat /tmp/result.log >> $OUTDIR/explain.txt 2>&1

				t=`grep 'Time' /tmp/result.log | tail -n 1 | awk '{print $2}'`

				echo $ts $r $workers $eic $p $t >> $OUTDIR/results.csv

			done

		done

	done

done

